/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Mozilla GNOME integration code.
 *
 * The Initial Developer of the Original Code is
 * Canonical Ltd.
 * Portions created by the Initial Developer are Copyright (C) 2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Chris Coulson <chris.coulson@canonical.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "nsGSettingsService.h"
#include "nsStringAPI.h"
#include "nsCOMPtr.h"
#include "prlink.h"
#include "nsComponentManagerUtils.h"

#include <glib.h>
#include <glib-object.h>

typedef struct _GSettings GSettings;

typedef GSettings * (*_g_settings_new_fn) (const char* schema);
typedef const char * const* (*_g_settings_list_schemas_fn) (void);
typedef gboolean (*_g_settings_set_string_fn) (GSettings* settings,
                                              const char* key,
                                              const char* value);
typedef gboolean (*_g_settings_set_boolean_fn) (GSettings* settings,
                                               const char* key,
                                               gboolean value);
typedef gboolean (*_g_settings_set_int_fn) (GSettings* settings,
                                            const char* key,
                                            int value);
typedef char * (*_g_settings_get_string_fn) (GSettings* settings,
                                            const char* key);
typedef gboolean (*_g_settings_get_boolean_fn) (GSettings* settings,
                                                const char* key);
typedef int (*_g_settings_get_int_fn) (GSettings* settings,
                                       const char* key);

static _g_settings_new_fn _g_settings_new;
static _g_settings_list_schemas_fn _g_settings_list_schemas;
static PRLibrary *gioLib = nsnull;

class nsGSettingsCollection : public nsIGSettingsCollection
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIGSETTINGSCOLLECTION

  nsGSettingsCollection(GSettings* aSettings) : mSettings(aSettings) {}
  ~nsGSettingsCollection() { g_object_unref(mSettings); }

private:
  GSettings *mSettings;
};

NS_IMPL_ISUPPORTS1(nsGSettingsCollection, nsIGSettingsCollection)

NS_IMETHODIMP
nsGSettingsCollection::SetString(const nsACString& aKey,
                                 const nsACString& aValue)
{
  _g_settings_set_string_fn _g_settings_set_string = 
    (_g_settings_set_string_fn) PR_FindFunctionSymbol(gioLib,
                                  "g_settings_set_string");
  if (!_g_settings_set_string)
    return NS_ERROR_FAILURE;

  gboolean res = _g_settings_set_string(mSettings,
                                        PromiseFlatCString(aKey).get(),
                                        PromiseFlatCString(aValue).get());

  return res ? NS_OK : NS_ERROR_FAILURE;
}

NS_IMETHODIMP
nsGSettingsCollection::SetBoolean(const nsACString& aKey,
                                  PRBool aValue)
{
  _g_settings_set_boolean_fn _g_settings_set_boolean =
    (_g_settings_set_boolean_fn) PR_FindFunctionSymbol(gioLib,
                                    "g_settings_set_boolean");
  if (!_g_settings_set_boolean)
    return NS_ERROR_FAILURE;

  gboolean res = _g_settings_set_boolean(mSettings,
                                         PromiseFlatCString(aKey).get(),
                                         aValue);

  return res ? NS_OK : NS_ERROR_FAILURE;
}

NS_IMETHODIMP
nsGSettingsCollection::SetInt(const nsACString& aKey,
                              PRInt32 aValue)
{
  _g_settings_set_int_fn _g_settings_set_int =
    (_g_settings_set_int_fn) PR_FindFunctionSymbol(gioLib,
                               "g_settings_set_int");
  if (!_g_settings_set_int)
    return NS_ERROR_FAILURE;

  gboolean res = _g_settings_set_int(mSettings,
                                     PromiseFlatCString(aKey).get(),
                                     aValue);

  return res ? NS_OK : NS_ERROR_FAILURE;
}

NS_IMETHODIMP
nsGSettingsCollection::GetString(const nsACString& aKey,
                                 nsACString& aResult)
{
  _g_settings_get_string_fn _g_settings_get_string =
    (_g_settings_get_string_fn) PR_FindFunctionSymbol(gioLib,
                                  "g_settings_get_string");
  if (!_g_settings_get_string)
    return NS_ERROR_FAILURE;

  char *result = _g_settings_get_string(mSettings,
                                        PromiseFlatCString(aKey).get());
  if (!result)
    return NS_ERROR_FAILURE;

  aResult.Assign(result);
  g_free(result);
  return NS_OK;
}

NS_IMETHODIMP
nsGSettingsCollection::GetBoolean(const nsACString& aKey,
                                  PRBool* aResult)
{
  _g_settings_get_boolean_fn _g_settings_get_boolean =
    (_g_settings_get_boolean_fn) PR_FindFunctionSymbol(gioLib,
                                  "g_settings_get_boolean");
  if (!_g_settings_get_boolean)
    return NS_ERROR_FAILURE;

  gboolean res = _g_settings_get_boolean(mSettings,
                                         PromiseFlatCString(aKey).get());
  *aResult = res ? PR_TRUE : PR_FALSE;

  return NS_OK;
}

NS_IMETHODIMP
nsGSettingsCollection::GetInt(const nsACString& aKey,
                              PRInt32* aResult)
{
  _g_settings_get_int_fn _g_settings_get_int =
    (_g_settings_get_int_fn) PR_FindFunctionSymbol(gioLib,
                                "g_settings_get_int");
  if (!_g_settings_get_int)
    return NS_ERROR_FAILURE;

  *aResult = _g_settings_get_int(mSettings,
                                 PromiseFlatCString(aKey).get());

  return NS_OK;
}

nsresult
nsGSettingsService::Init()
{
  if (!gioLib) {
    gioLib = PR_LoadLibrary("libgio-2.0.so.0");
  }

  _g_settings_new = (_g_settings_new_fn)
    PR_FindFunctionSymbol(gioLib, "g_settings_new");
  _g_settings_list_schemas = (_g_settings_list_schemas_fn)
    PR_FindFunctionSymbol(gioLib, "g_settings_list_schemas");

  if (!_g_settings_new || !_g_settings_list_schemas) {
    PR_UnloadLibrary(gioLib);
    return NS_ERROR_FAILURE;
  }

  return NS_OK;
}

NS_IMPL_ISUPPORTS1(nsGSettingsService, nsIGSettingsService)

nsGSettingsService::~nsGSettingsService()
{
  if (gioLib) {
    PR_UnloadLibrary(gioLib);
    gioLib = nsnull;
  }
}

NS_IMETHODIMP
nsGSettingsService::GetCollectionForSchema(const nsACString& schema,
                                           nsIGSettingsCollection** collection)
{
  const char * const *schemas = _g_settings_list_schemas();
  unsigned int i = 0;

  for (i = 0; schemas[i] != NULL; i++) {
    if (strcmp(schemas[i], PromiseFlatCString(schema).get()) == 0) {
      GSettings *settings = _g_settings_new(PromiseFlatCString(schema).get());
      nsGSettingsCollection *mozGSettings = new nsGSettingsCollection(settings);
      NS_ADDREF(*collection = mozGSettings);
      return NS_OK;
    }
  }

  return NS_ERROR_FAILURE;
}
